@@ -25,13 +25,18 @@ import java.io.InputStream;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
+import java.lang.reflect.Field;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import android.annotation.TargetApi;
 import android.app.Activity;
+import android.app.ActivityManager.TaskDescription;
 import android.app.AlertDialog;
 import android.app.Dialog;
 import android.app.Notification;
@@ -44,13 +49,22 @@ import android.content.SharedPreferences;
 import android.content.pm.ApplicationInfo;
 import android.content.pm.PackageManager;
 import android.content.pm.PackageManager.NameNotFoundException;
+import android.content.res.Configuration;
 import android.content.res.Resources;
+import android.graphics.Bitmap;
+import android.graphics.BitmapFactory;
+import android.graphics.Color;
+import android.graphics.Point;
+import android.Manifest;
 import android.media.AudioManager;
 import android.net.Uri;
+import android.os.Build;
 import android.os.Bundle;
 import android.os.Environment;
 import android.os.Message;
 import android.os.PowerManager;
+import android.support.v4.app.ActivityCompat;
+import android.support.v4.content.ContextCompat;
 import android.text.SpannableString;
 import android.text.method.LinkMovementMethod;
 import android.text.util.Linkify;
@@ -59,12 +73,14 @@ import android.util.Log;
 import android.view.Display;
 import android.view.Menu;
 import android.view.MenuItem;
+import android.view.View;
 import android.view.Window;
 import android.view.WindowManager;
 import android.view.inputmethod.InputMethodManager;
 import android.widget.RelativeLayout;
 import android.widget.TextView;
 import android.widget.Toast;
+import android.support.v4.app.NotificationCompat;
 
 
 public class Navit extends Activity
@@ -75,6 +91,11 @@ public class Navit extends Activity
 	private NavitActivityResult      ActivityResults[];
 	public static InputMethodManager mgr                            = null;
 	public static DisplayMetrics     metrics                        = null;
+	public static int                status_bar_height              = 0;
+	public static int                action_bar_default_height      = 0;
+	public static int                navigation_bar_height          = 0;
+	public static int                navigation_bar_height_landscape= 0;
+	public static int                navigation_bar_width           = 0;
 	public static Boolean            show_soft_keyboard             = false;
 	public static Boolean            show_soft_keyboard_now_showing = false;
 	public static long               last_pressed_menu_key          = 0L;
@@ -97,7 +118,24 @@ public class Navit extends Activity
 	static final String              NAVIT_DATA_SHARE_DIR           = NAVIT_DATA_DIR + "/share";
 	static final String              FIRST_STARTUP_FILE             = NAVIT_DATA_SHARE_DIR + "/has_run_once.txt";
 	public static final String       NAVIT_PREFS                    = "NavitPrefs";
+	Boolean                          isFullscreen                   = false;
+	private static final int         MY_PERMISSIONS_REQUEST_ALL     = 101;
+	public static NotificationManager       nm;
 
+	
+	/**
+	 * @brief A Runnable to restore soft input when the user returns to the activity.
+	 * 
+	 * An instance of this class can be passed to the main message queue in the Activity's
+	 * {@code onRestore()} method.
+	 */
+	private class SoftInputRestorer implements Runnable {
+		public void run() {
+			Navit.this.showNativeKeyboard();
+		}
+	}
+	
+	
 	public void removeFileIfExists(String source) {
 		File file = new File(source);
 
@@ -138,7 +176,7 @@ public class Navit extends Activity
 	}
 
 
-	public static String _(String in)
+	public static String T(String in)
 	{
 		return NavitTextTranslations.get_text(in);
 	}
@@ -202,18 +240,8 @@ public class Navit extends Activity
 			AlertDialog.Builder infobox = new AlertDialog.Builder(this);
 			infobox.setTitle(getString(R.string.initial_info_box_title)); // TRANS
 			infobox.setCancelable(false);
-			final TextView message = new TextView(this);
-			message.setFadingEdgeLength(20);
-			message.setVerticalFadingEdgeEnabled(true);
-			// message.setVerticalScrollBarEnabled(true);
-			RelativeLayout.LayoutParams rlp = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.FILL_PARENT, RelativeLayout.LayoutParams.FILL_PARENT);
-	
-			message.setLayoutParams(rlp);
-			final SpannableString s = new SpannableString(getString(R.string.initial_info_box_message)); // TRANS
-			Linkify.addLinks(s, Linkify.WEB_URLS);
-			message.setText(s);
-			message.setMovementMethod(LinkMovementMethod.getInstance());
-			infobox.setView(message);
+			
+			infobox.setMessage(R.string.initial_info_box_message);
 	
 			// TRANS
 			infobox.setPositiveButton(getString(R.string.initial_info_box_OK), new DialogInterface.OnClickListener() {
@@ -246,6 +274,8 @@ public class Navit extends Activity
 		super.onCreate(savedInstanceState);
 		if (android.os.Build.VERSION.SDK_INT < android.os.Build.VERSION_CODES.HONEYCOMB)
 			this.requestWindowFeature(Window.FEATURE_NO_TITLE);
+		else
+			this.getActionBar().hide();
 
 		dialogs = new NavitDialogs(this);
 
@@ -264,13 +294,40 @@ public class Navit extends Activity
 		// NOTIFICATION
 		// Setup the status bar notification		
 		// This notification is removed in the exit() function
-		NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);	// Grab a handle to the NotificationManager
-		Notification NavitNotification = new Notification(R.drawable.icon, getString(R.string.notification_ticker), System.currentTimeMillis());	// Create a new notification, with the text string to show when the notification first appears
+		nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);	// Grab a handle to the NotificationManager
 		PendingIntent appIntent = PendingIntent.getActivity(getApplicationContext(), 0, getIntent(), 0);
-		NavitNotification.setLatestEventInfo(getApplicationContext(), "Navit", getString(R.string.notification_event_default), appIntent);	// Set the text in the notification
-		NavitNotification.flags|=Notification.FLAG_ONGOING_EVENT;	// Ensure that the notification appears in Ongoing
-		nm.notify(R.string.app_name, NavitNotification);	// Set the notification
 		
+		NotificationCompat.Builder builder = new NotificationCompat.Builder(this);
+		builder.setContentIntent(appIntent);
+		builder.setAutoCancel(false).setOngoing(true);
+		builder.setContentTitle(getString(R.string.app_name));
+		builder.setContentText(getString(R.string.notification_event_default));
+		builder.setSmallIcon(R.drawable.ic_notify);
+		Notification NavitNotification = builder.build();
+		nm.notify(R.string.app_name, NavitNotification);// Show the notification
+		
+		// Status and navigation bar sizes
+		// These are platform defaults and do not change with rotation, but we have to figure out which ones apply
+		// (is the navigation bar visible? on the side or at the bottom?)
+		Resources resources = getResources();
+		int shid = resources.getIdentifier("status_bar_height", "dimen", "android");
+		int adhid = resources.getIdentifier("action_bar_default_height", "dimen", "android");
+		int nhid = resources.getIdentifier("navigation_bar_height", "dimen", "android");
+		int nhlid = resources.getIdentifier("navigation_bar_height_landscape", "dimen", "android");
+		int nwid = resources.getIdentifier("navigation_bar_width", "dimen", "android");
+		status_bar_height = (shid > 0) ? resources.getDimensionPixelSize(shid) : 0;
+		action_bar_default_height = (adhid > 0) ? resources.getDimensionPixelSize(adhid) : 0;
+		navigation_bar_height = (nhid > 0) ? resources.getDimensionPixelSize(nhid) : 0;
+		navigation_bar_height_landscape = (nhlid > 0) ? resources.getDimensionPixelSize(nhlid) : 0;
+		navigation_bar_width = (nwid > 0) ? resources.getDimensionPixelSize(nwid) : 0;
+		Log.d(TAG, String.format("status_bar_height=%d, action_bar_default_height=%d, navigation_bar_height=%d, navigation_bar_height_landscape=%d, navigation_bar_width=%d", 
+				status_bar_height, action_bar_default_height, navigation_bar_height, navigation_bar_height_landscape, navigation_bar_width));
+		if ((ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED)||
+				(ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED)) {
+			Log.d (TAG,"ask for permission(s)");
+			ActivityCompat.requestPermissions(this,
+				new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE,Manifest.permission.ACCESS_FINE_LOCATION},MY_PERMISSIONS_REQUEST_ALL);
+		}
 		// get the local language -------------
 		Locale locale = java.util.Locale.getDefault();
 		String lang = locale.getLanguage();
@@ -380,7 +437,13 @@ public class Navit extends Activity
 	public void onResume()
 	{
 		super.onResume();
-		Log.e("Navit", "OnResume");
+		Log.d("Navit", "OnResume");
+		if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP) {
+			/* Required to make system bars fully transparent */
+			getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_STABLE
+                    | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
+                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION);
+		}
 		//InputMethodManager mgr = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
 		// DEBUG
 		// intent_data = "google.navigation:q=Wien Burggasse 27";
@@ -402,8 +465,61 @@ public class Navit extends Activity
 				Log.e("Navit", "timestamp for navigate_to expired! not using data");
 			}
 		}
+		Log.d(TAG, "onResume");
+		if (show_soft_keyboard_now_showing) {
+			/* Calling showNativeKeyboard() directly won't work here, we need to use the message queue */
+			View cf = getCurrentFocus();
+			if (cf == null)
+				Log.e(TAG, "no view in focus, can't get a handler");
+			else
+				cf.getHandler().post(new SoftInputRestorer());
+		}
 	}
-
+	
+	@Override
+	public void onPause() {
+		super.onPause();
+		Log.d(TAG, "onPause");
+		if (show_soft_keyboard_now_showing) {
+			Log.d(TAG, "onPause:hiding soft input");
+			this.hideNativeKeyboard();
+			show_soft_keyboard_now_showing = true;
+		}
+	}
+	
+	@Override
+	public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
+		switch (requestCode) {
+			case MY_PERMISSIONS_REQUEST_ALL: {
+				if (grantResults.length > 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED
+						&& grantResults[1] == PackageManager.PERMISSION_GRANTED) {
+						// ok, we got permissions
+				} else {
+					AlertDialog.Builder infobox = new AlertDialog.Builder(this);
+					infobox.setTitle(getString(R.string.permissions_info_box_title)); // TRANS
+					infobox.setCancelable(false);
+					final TextView message = new TextView(this);
+					message.setFadingEdgeLength(20);
+					message.setVerticalFadingEdgeEnabled(true);
+					RelativeLayout.LayoutParams rlp = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.FILL_PARENT, RelativeLayout.LayoutParams.FILL_PARENT);
+					message.setLayoutParams(rlp);
+					final SpannableString s = new SpannableString(getString(R.string.permissions_not_granted)); // TRANS
+					message.setText(s);
+					message.setMovementMethod(LinkMovementMethod.getInstance());
+					infobox.setView(message);
+					// TRANS
+					infobox.setPositiveButton(getString(R.string.initial_info_box_OK), new DialogInterface.OnClickListener() {
+						public void onClick(DialogInterface arg0, int arg1) {
+							exit();
+						}
+					});
+					infobox.show();
+				}
+				return;
+			}
+		}
+	}
+	
 	private void parseNavigationURI(String schemeSpecificPart) {
 		String naviData[]= schemeSpecificPart.split("&");
 		Pattern p = Pattern.compile("(.*)=(.*)");
@@ -463,6 +579,10 @@ public class Navit extends Activity
 		ActivityResults[requestCode] = ActivityResult;
 	}
 
+	/*
+	 * This is unused since we dont have the dropdown 
+	 * TODO: recheck if this is right and remove this!
+	 */
 	@Override
 	public boolean onPrepareOptionsMenu(Menu menu)
 	{
@@ -556,13 +676,19 @@ public class Navit extends Activity
 				startActivityForResult(map_download_list_activity, Navit.NavitDownloaderSelectMap_id);
 				break;
 			case 5 :
-				// toggle the normal POI layers (to avoid double POIs)
+				// toggle the normal POI layers and labels (to avoid double POIs)
 				Message msg = Message.obtain(N_NavitGraphics.callback_handler, NavitGraphics.msg_type.CLB_CALL_CMD.ordinal());
 				Bundle b = new Bundle();
 				b.putString("cmd", "toggle_layer(\"POI Symbols\");");
 				msg.setData(b);
 				msg.sendToTarget();
 
+				msg = Message.obtain(N_NavitGraphics.callback_handler, NavitGraphics.msg_type.CLB_CALL_CMD.ordinal());
+				b = new Bundle();
+				b.putString("cmd", "toggle_layer(\"POI Labels\");");
+				msg.setData(b);
+				msg.sendToTarget();
+
 				// toggle full POI icons on/off
 				msg = Message.obtain(N_NavitGraphics.callback_handler, NavitGraphics.msg_type.CLB_CALL_CMD.ordinal());
 				b = new Bundle();
@@ -590,7 +716,68 @@ public class Navit extends Activity
 				break;
 		}
 	}
+	
+	
+	/**
+	 * @brief Shows the Options menu.
+	 * 
+	 * Calling this method has the same effect as pressing the hardware Menu button, where present, or touching
+	 * the overflow button in the Action bar.
+	 */
+	public void showMenu() {
+		openOptionsMenu();
+	}
 
+	
+	/**
+	 * @brief Shows the native keyboard or other input method.
+	 * 
+	 * @return {@code true} if an input method is going to be displayed, {@code false} if not
+	 */
+	public int showNativeKeyboard() {
+		/*
+		 * Apologies for the huge mess that this function is, but Android's soft input API is a big
+		 * nightmare. Its devs have mercifully given us an option to show or hide the keyboard, but
+		 * there is no reliable way to figure out if it is actually showing, let alone how much of the
+		 * screen it occupies, so our best bet is guesswork.
+		 */
+		Configuration config = getResources().getConfiguration();
+		if ((config.keyboard == Configuration.KEYBOARD_QWERTY) && (config.hardKeyboardHidden == Configuration.HARDKEYBOARDHIDDEN_NO))
+			/* physical keyboard present, exit */
+			return 0;
+		
+		/* Use SHOW_FORCED here, else keyboard won't show in landscape mode */
+		mgr.showSoftInput(getCurrentFocus(), InputMethodManager.SHOW_FORCED);
+		show_soft_keyboard_now_showing = true;
+
+		/* 
+		 * Crude way to estimate the height occupied by the keyboard: for AOSP on KitKat and Lollipop it
+		 * is about 62-63% of available screen width (in portrait mode) but no more than slightly above
+		 * 46% of height (in landscape mode).
+		 */
+		Display display_ = getWindowManager().getDefaultDisplay();
+		int width_ = display_.getWidth();
+		int height_ = display_.getHeight();
+		int maxHeight = height_ * 47 / 100;
+		int inputHeight = width_ * 63 / 100;
+		if (inputHeight > (maxHeight))
+			inputHeight = maxHeight;
+
+		/* the receiver isn't going to fire before the UI thread becomes idle, well after this method returns */
+		Log.d(TAG, "showNativeKeyboard:return (assuming true)");
+		return inputHeight;
+	}
+	
+	
+	/**
+	 * @brief Hides the native keyboard or other input method.
+	 */
+	public void hideNativeKeyboard() {
+		mgr.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), 0);
+		show_soft_keyboard_now_showing = false;
+	}
+	
+	
 	void setDestination(float latitude, float longitude, String address) {
 		Toast.makeText( getApplicationContext(),getString(R.string.address_search_set_destination) + "\n" + address, Toast.LENGTH_LONG).show(); //TRANS
 
@@ -637,7 +824,7 @@ public class Navit extends Activity
 				SharedPreferences.Editor  prefs_editor = prefs.edit(); 
 				prefs_editor.putString("filenamePath", newDir); 
 				prefs_editor.commit();
-				Toast.makeText(this, String.format(Navit._("New location set to %s\nRestart Navit to apply the changes."),newDir),Toast.LENGTH_LONG).show();                 
+				Toast.makeText(this, String.format(Navit.T("New location set to %s\nRestart Navit to apply the changes."),newDir),Toast.LENGTH_LONG).show();                 
 			} 
 				else Log.w(TAG, "select path failed"); 
 			break; 			
@@ -690,25 +877,33 @@ public class Navit extends Activity
 		NavitDestroy();
 	}
 	
-	public void fullscreen(int fullscreen) 
-	{
-		if(fullscreen != 0) 
-		{
+	public void fullscreen(int fullscreen) {
+		int w, h;
+		
+		isFullscreen = (fullscreen != 0);
+		if (isFullscreen) {
 			getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
 			getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
-			if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.HONEYCOMB)
-				this.getActionBar().hide();
 		}
-		else 
-		{
+		else {
 			getWindow().addFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
 			getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
-			if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.HONEYCOMB)
-				this.getActionBar().show();
 		}
+		
+		Display display_ = getWindowManager().getDefaultDisplay();
+		if (Build.VERSION.SDK_INT < 17) {
+			w = display_.getWidth();
+			h = display_.getHeight();
+		} else {
+			Point size = new Point();
+			display_.getRealSize(size);
+			w = size.x;
+			h = size.y;
+		}
+		Log.d(TAG, String.format("Toggle fullscreen, w=%d, h=%d", w, h));
+		N_NavitGraphics.handleResize(w, h);
 	}
 
-
 	public void disableSuspend()
 	{
 		wl.acquire();
@@ -717,8 +912,7 @@ public class Navit extends Activity
 
 	public void exit()
 	{
-		NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
-		nm.cancel(R.string.app_name);
+		nm.cancelAll();
 		NavitVehicle.removeListener();
 		NavitDestroy();
 	}
