@@ -605,7 +605,8 @@ attr_generic_get_attr(struct attr **attrs, struct attr **def_attrs, enum attr_ty
  * contain an attribute whose type matches that of the new one, the new
  * attribute is inserted into the list.
  *
- * @param attrs Points to the array of attribute pointers to be updated
+ * @param attrs Points to the array of attribute pointers to be updated (if NULL, this function will
+ * create a new list containing only the new attribute)
  * @param attr The new attribute.
  * @return Pointer to the updated attribute list
  */
@@ -689,6 +690,19 @@ attr_generic_prepend_attr(struct attr **attrs, struct attr *attr)
 	return curr;
 }
 
+/**
+ * @brief Removes an attribute from an attribute list.
+ *
+ * If `attrs` contains `attr`, a new attribute list is created (which contains all attributes, except
+ * for `attr`) and both `attrs` (the original attribute list) and `attr` are freed.
+ *
+ * If `attrs` does not contain `attr`, this function is a no-op.
+ *
+ * @param attrs The attribute list
+ * @param attr The attribute to remove from the list
+ *
+ * @return The new attribute list
+ */
 struct attr **
 attr_generic_remove_attr(struct attr **attrs, struct attr *attr)
 {
@@ -835,7 +849,8 @@ attr_free_content_do(struct attr *attr)
 			obj->func->unref(obj);
 	}
 	if (!(attr->type >= attr_type_int_begin && attr->type <= attr_type_int_end) && 
-	    !(attr->type >= attr_type_object_begin && attr->type <= attr_type_object_end))
+	    !(attr->type >= attr_type_object_begin && attr->type <= attr_type_object_end) &&
+	    attr->type != attr_item_type)
 		g_free(attr->u.data);
 }
 
@@ -860,6 +875,8 @@ attr_dup_content(struct attr *src, struct attr *dst)
 	dst->type=src->type;
 	if (src->type >= attr_type_int_begin && src->type <= attr_type_int_end) 
 		dst->u.num=src->u.num;
+	else if (src->type == attr_item_type)
+		dst->u.item_type=src->u.item_type;
 	else if (src->type >= attr_type_object_begin && src->type <= attr_type_object_end) {
 		if (HAS_OBJECT_FUNC(src->type)) {
 			struct navit_object *obj=src->u.data;
@@ -886,6 +903,15 @@ attr_dup(struct attr *attr)
 	return ret;
 }
 
+/**
+ * @brief Frees a list of attributes.
+ *
+ * This frees the pointer array as well as the attributes referenced by the pointers.
+ *
+ * It is safe to call this function with a NULL argument; doing so is a no-op.
+ *
+ * @param attrs The attribute list to free
+ */
 void
 attr_list_free(struct attr **attrs)
 {
@@ -896,6 +922,17 @@ attr_list_free(struct attr **attrs)
 	g_free(attrs);
 }
 
+/**
+ * @brief Duplicates a list of attributes.
+ *
+ * This creates a deep copy, i.e. the attributes in the list are copied as well.
+ *
+ * It is safe to call this function with a NULL argument; in this case, NULL will be returned.
+ *
+ * @param attrs The attribute list to copy
+ *
+ * @return The copy of the attribute list
+ */
 struct attr **
 attr_list_dup(struct attr **attrs)
 {
