@@ -1350,12 +1350,6 @@ request_navit_zoom(DBusConnection *connection, DBusMessage *message)
 
 }
 
-/**
- * @brief Exports the current route as a GPX file
- * @param connection The DBusConnection object through which a message arrived
- * @param message The DBusMessage including the 'filename' parameter
- * @returns An empty reply if everything went right, otherwise DBUS_HANDLER_RESULT_NOT_YET_HANDLED
- */
 static DBusHandlerResult
 request_navit_zoom_to_route(DBusConnection *connection, DBusMessage *message)
 {
@@ -1375,6 +1369,13 @@ request_navit_zoom_to_route(DBusConnection *connection, DBusMessage *message)
 
 }
 
+/**
+ * @brief Exports the current route as a GPX file.
+ *
+ * @param connection The DBusConnection object through which a message arrived
+ * @param message The DBusMessage including the `filename` parameter
+ * @returns An empty reply if everything went right, otherwise `DBUS_HANDLER_RESULT_NOT_YET_HANDLED`
+ */
 static DBusHandlerResult
 request_navit_route_export_gpx(DBusConnection *connection, DBusMessage *message)
 {
@@ -1722,7 +1723,7 @@ request_search_list_destroy(DBusConnection *connection, DBusMessage *message)
 	return request_destroy(connection, message, "search_list", NULL, (void (*)(void *)) search_list_destroy);
 }
 
-void
+static void
 request_search_list_common(struct search_list_common *slc, DBusMessageIter *iter4)
 {
 	if (slc->postal)
