@@ -621,7 +621,7 @@ result_op(struct context *ctx, enum op_type op_type, const char *op, struct resu
 	default:
 		break;
 	}
-	dbg(lvl_error,"Unkown op %d %s\n",op_type,op);
+	dbg(lvl_error,"Unknown op %d %s\n",op_type,op);
 	ctx->error=internal;
 }
 
@@ -917,7 +917,7 @@ command_call_function(struct context *ctx, struct result *res)
 				callback_list_call_attr_4(cbl.u.callback_list, attr_command, function, list, &out, &valid);
 				if (valid!=1){
 					dbg(lvl_error, "invalid command ignored: \"%s\"; see http://wiki.navit-project.org/index.php/"
-					    "The_Navit_Command_Interface for valid commands.\n", function);
+					    "OSD#Navit_commands for valid commands.\n", function);
 				}
 				if (out && out[0]) {
 					result_free(res);
