@@ -36,6 +36,10 @@
 #include "gui_internal_search.h"
 #include "gui_internal_poi.h"
 #include "gui_internal_command.h"
+#if HAS_IFADDRS
+#include <ifaddrs.h>
+#include <arpa/inet.h>
+#endif
 
 extern char *version;
 
@@ -294,6 +298,8 @@ gui_internal_cmd_enter_coord(struct gui_priv *this, char *function, struct attr
 
 	if (this->keyboard)
 		gui_internal_widget_append(w, gui_internal_keyboard(this, VKBD_DEGREE));
+	else
+		gui_internal_keyboard_show_native(this, w, VKBD_DEGREE, NULL);
 	gui_internal_menu_render(this);
 }
 
@@ -444,13 +450,21 @@ gui_internal_cmd2_setting_layout(struct gui_priv *this, char *function, struct a
 	navit_attr_iter_destroy(iter);
 	gui_internal_menu_render(this);
 }
+
+/*
+ * @brief Displays Route Height Profile
+ *
+ * displays a heightprofile if a route is active and 
+ * some heightinfo is provided by means of a map
+ *
+ * the name of the file providing the heightlines must
+ * comply with *.heightlines.bin
+ * 
+ */
 static void
 gui_internal_cmd2_route_height_profile(struct gui_priv *this, char *function, struct attr **in, struct attr ***out, int *valid)
 {
-
-
 	struct widget * menu, *box;
-
 	struct map * map=NULL;
 	struct map_rect * mr=NULL;
 	struct route * route;
@@ -458,17 +472,21 @@ gui_internal_cmd2_route_height_profile(struct gui_priv *this, char *function, st
 	struct mapset *ms;
 	struct mapset_handle *msh;
 	int x,i,first=1,dist=0;
+	int diagram_points_count = 0;
 	struct coord c,last,res;
 	struct coord_rect rbbox,dbbox;
 	struct map_selection sel;
 	struct heightline *heightline,*heightlines=NULL;
 	struct diagram_point *min,*diagram_point,*diagram_points=NULL;
+	struct point p[2];
+	int min_ele=INT_MAX;
+	int max_ele=INT_MIN;
+	int distance=0;
 	sel.next=NULL;
 	sel.order=18;
 	sel.range.min=type_height_line_1;
 	sel.range.max=type_height_line_3;
 
-
 	menu=gui_internal_menu(this,_("Height Profile"));
 	box = gui_internal_box_new(this, gravity_left_top| orientation_vertical | flags_fill | flags_expand);
 	gui_internal_widget_append(menu, box);
@@ -489,11 +507,24 @@ gui_internal_cmd2_route_height_profile(struct gui_priv *this, char *function, st
 			}
 		}
 		map_rect_destroy(mr);
+		mr = NULL;
 		ms=navit_get_mapset(this->nav);
 		if (!first && ms) {
+			int heightmap_installed = FALSE;
 			msh=mapset_open(ms);
 			while ((map=mapset_next(msh, 1))) {
-				mr=map_rect_new(map, &sel);
+				struct attr data_attr;
+				if (map_get_attr(map, attr_data, &data_attr, NULL)){
+					dbg(lvl_debug,"map name = %s\n",data_attr.u.str);
+					if (strstr(data_attr.u.str,".heightlines.bin")){
+						dbg(lvl_info,"reading heightlines from map %s\n",data_attr.u.str);
+						mr=map_rect_new(map, &sel);
+						heightmap_installed = TRUE;
+					}
+					else {
+						dbg(lvl_debug,"ignoring map %s\n",data_attr.u.str);
+					}
+				}
 				if (mr) {
 					while((item = map_rect_get_item(mr))) {
 						if (item->type >= sel.range.min && item->type <= sel.range.max) {
@@ -505,9 +536,20 @@ gui_internal_cmd2_route_height_profile(struct gui_priv *this, char *function, st
 						}
 					}
 					map_rect_destroy(mr);
+					mr = NULL;
 				}
 			}
 			mapset_close(msh);
+			if (!heightmap_installed){
+				char *text;
+				struct widget *w;
+				text=g_strdup_printf("%s",_("please install a map *.heightlines.bin to provide elevationdata"));
+				gui_internal_widget_append(box, w=gui_internal_label_new(this, text));
+				w->flags=gravity_bottom_center|orientation_horizontal|flags_fill;
+				g_free(text);
+				gui_internal_menu_render(this);
+				return;
+			}
 		}
 	}
 	map=NULL;
@@ -537,7 +579,11 @@ gui_internal_cmd2_route_height_profile(struct gui_priv *this, char *function, st
 										diagram_point->c.y=heightline->height;
 										diagram_point->next=diagram_points;
 										diagram_points=diagram_point;
-										dbg(lvl_info,"%d %d\n", diagram_point->c.x, diagram_point->c.y);
+										diagram_points_count ++;
+										dbg(lvl_debug,"%d %d\n", diagram_point->c.x, diagram_point->c.y);
+										max_ele=fmax(max_ele, diagram_point->c.y);
+										min_ele=fmin(min_ele, diagram_point->c.y);
+										distance=diagram_point->c.x;
 									}
 								}
 							}
@@ -548,18 +594,30 @@ gui_internal_cmd2_route_height_profile(struct gui_priv *this, char *function, st
 				}
 				last=c;
 			}
-
 		}
 	}
-
+	while (heightlines){
+		heightline=heightlines;
+		heightlines=heightlines->next;
+		g_free(heightline);
+	}
 	if(mr)
 		map_rect_destroy(mr);
 
-	gui_internal_menu_render(this);
-
-	if(!diagram_points) 
+	if(diagram_points_count < 2){
+		char *text;
+		struct widget *w;
+		text=g_strdup_printf("%s",_("The route must cross at least 2 heightlines"));
+		gui_internal_widget_append(box, w=gui_internal_label_new(this, text));
+		w->flags=gravity_bottom_center|orientation_horizontal|flags_fill;
+		g_free(text);
+		gui_internal_menu_render(this);
+		if(diagram_points)
+			g_free(diagram_points);
 		return;
+	}
 
+	gui_internal_menu_render(this);
 	first=1;
 	diagram_point=diagram_points;
 	while (diagram_point) {
@@ -571,38 +629,53 @@ gui_internal_cmd2_route_height_profile(struct gui_priv *this, char *function, st
 			coord_rect_extend(&dbbox, &diagram_point->c);
 		diagram_point=diagram_point->next;
 	}
-	dbg(lvl_info,"%d %d %d %d\n", dbbox.lu.x, dbbox.lu.y, dbbox.rl.x, dbbox.rl.y);
+	dbg(lvl_debug,"%d %d %d %d\n", dbbox.lu.x, dbbox.lu.y, dbbox.rl.x, dbbox.rl.y);
 	if (dbbox.rl.x > dbbox.lu.x && dbbox.lu.x*100/(dbbox.rl.x-dbbox.lu.x) <= 25)
 		dbbox.lu.x=0;
 	if (dbbox.lu.y > dbbox.rl.y && dbbox.rl.y*100/(dbbox.lu.y-dbbox.rl.y) <= 25)
 		dbbox.rl.y=0;
-	dbg(lvl_info,"%d,%d %dx%d\n", box->p.x, box->p.y, box->w, box->h);
+	dbg(lvl_debug,"%d,%d %dx%d\n", box->p.x, box->p.y, box->w, box->h);
 	x=dbbox.lu.x;
 	first=1;
-	for (;;) {
-		struct point p[2];
-		min=NULL;
-		diagram_point=diagram_points;
-		while (diagram_point) {
-			if (diagram_point->c.x >= x && (!min || min->c.x > diagram_point->c.x))
-				min=diagram_point;
-			diagram_point=diagram_point->next;
+	if (diagram_points_count > 1 && dbbox.rl.x != dbbox.lu.x && dbbox.lu.y != dbbox.rl.y){
+		for (;;) {
+			min=NULL;
+			diagram_point=diagram_points;
+			while (diagram_point) {
+				if (diagram_point->c.x >= x && (!min || min->c.x > diagram_point->c.x))
+					min=diagram_point;
+				diagram_point=diagram_point->next;
+			}
+			if (! min)
+				break;
+			p[1].x=(min->c.x-dbbox.lu.x)*(box->w-10)/(dbbox.rl.x-dbbox.lu.x)+box->p.x+5;
+			p[1].y=(box->h)-5-(min->c.y-dbbox.rl.y)*(box->h-10)/(dbbox.lu.y-dbbox.rl.y)+box->p.y;
+			dbg(lvl_debug,"%d,%d=%d,%d\n",min->c.x, min->c.y, p[1].x,p[1].y);
+			graphics_draw_circle(this->gra, this->foreground, &p[1], 2);
+			if (first)
+				first=0;
+			else
+				graphics_draw_lines(this->gra, this->foreground, p, 2);
+			p[0]=p[1];
+			x=min->c.x+1;
 		}
-		if (! min)
-			break;
-		p[1].x=(min->c.x-dbbox.lu.x)*(box->w-10)/(dbbox.rl.x-dbbox.lu.x)+box->p.x+5;
-		p[1].y=(min->c.y-dbbox.rl.y)*(box->h-10)/(dbbox.lu.y-dbbox.rl.y)+box->p.y+5;
-		dbg(lvl_info,"%d,%d=%d,%d\n",min->c.x, min->c.y, p[1].x,p[1].y);
-		graphics_draw_circle(this->gra, this->foreground, &p[1], 2);
-		if (first)
-			first=0;
-		else
-			graphics_draw_lines(this->gra, this->foreground, p, 2);
-		p[0]=p[1];
-		x=min->c.x+1;
 	}
 
+	struct point pTopLeft={0, box->p.y + 10};
+	struct point pBottomLeft={0, box->h + box->p.y - 2};
+	struct point pBottomRight={box->w - 100, box->h + box->p.y - 2};
+	char* minele_text=g_strdup_printf("%d m", min_ele);
+	char* maxele_text=g_strdup_printf("%d m", max_ele);
+	char* distance_text=g_strdup_printf("%.3f km", distance/1000.0);
+	graphics_draw_text_std(this->gra, 10, maxele_text, &pTopLeft);
+	graphics_draw_text_std(this->gra, 10, minele_text, &pBottomLeft);
+	graphics_draw_text_std(this->gra, 10, distance_text, &pBottomRight);
 
+	while (diagram_points){
+		diagram_point=diagram_points;
+		diagram_points=diagram_points->next;
+		g_free(diagram_point);
+	}
 }
 
 static void
@@ -724,6 +797,56 @@ gui_internal_cmd2_locale(struct gui_priv *this, char *function, struct attr **in
 	graphics_draw_mode(this->gra, draw_mode_end);
 }
 
+/**
+ * @brief display basic networking information
+ *
+ * @return nothing
+ *
+ * This function displays basic networking information, currently
+ * only the interface name and the associated IP address(es).
+ * Currently only works on non Windows systems.
+ *
+ */
+static void
+gui_internal_cmd2_network_info(struct gui_priv *this, char *function, struct attr **in, struct attr ***out, int *valid)
+{
+#if HAS_IFADDRS
+	struct widget *menu,*wb,*w;
+	char *text;
+
+	graphics_draw_mode(this->gra, draw_mode_begin);
+	menu=gui_internal_menu(this, _("Network info"));
+	menu->spx=this->spacing*10;
+	wb=gui_internal_box_new(this, gravity_top_center|orientation_vertical|flags_expand|flags_fill);
+	gui_internal_widget_append(menu, wb);
+
+	struct ifaddrs *addrs, *tmp;
+	getifaddrs(&addrs);
+	tmp = addrs;
+
+	while (tmp)
+	{
+		if (tmp->ifa_addr && tmp->ifa_addr->sa_family == AF_INET)
+		{
+			struct sockaddr_in *pAddr = (struct sockaddr_in *)tmp->ifa_addr;
+			if(g_ascii_strncasecmp(tmp->ifa_name,"lo",2 ) ) {
+				text=g_strdup_printf("%s: %s", tmp->ifa_name, inet_ntoa(pAddr->sin_addr));
+				gui_internal_widget_append(wb, w=gui_internal_label_new(this, text));
+				w->flags=gravity_bottom_center|orientation_horizontal|flags_fill;
+				g_free(text);
+			}
+		}
+		tmp = tmp->ifa_next;
+	}
+	freeifaddrs(addrs);
+
+	gui_internal_menu_render(this);
+	graphics_draw_mode(this->gra, draw_mode_end);
+#else
+	dbg(lvl_error, "Cannot show network info: ifaddr.h not found\n");
+#endif
+}
+
 static void
 gui_internal_cmd_formerdests(struct gui_priv *this, char *function, struct attr **in, struct attr ***out, int *valid)
 {
@@ -860,6 +983,8 @@ gui_internal_cmd_log(struct gui_priv *this)
 	gui_internal_widget_append(w, wl);
 	if (this->keyboard)
 		gui_internal_widget_append(w, gui_internal_keyboard(this, VKBD_FLAG_2 | gui_internal_keyboard_init_mode(getenv("LANG"))));
+	else
+		gui_internal_keyboard_show_native(this, w, VKBD_FLAG_2 | gui_internal_keyboard_init_mode(getenv("LANG")), getenv("LANG"));
 	gui_internal_menu_render(this);
 	gui_internal_leave(this);
 }
@@ -1075,7 +1200,6 @@ error:
 	return;
 }
 
-
 static void
 gui_internal_cmd_img(struct gui_priv * this, char *function, struct attr **in, struct attr ***out, int *valid)
 {
@@ -1167,6 +1291,8 @@ gui_internal_cmd2(struct gui_priv *this, char *function, struct attr **in, struc
 		gui_internal_cmd_formerdests(this, function, in, out, valid);
 	else if(!strcmp(function, "locale"))
 		gui_internal_cmd2_locale(this, function, in, out, valid);
+	else if(!strcmp(function, "network_info"))
+		gui_internal_cmd2_network_info(this, function, in, out, valid);
 	else if(!strcmp(function, "position"))
 		gui_internal_cmd2_position(this, function, in, out, valid);
 	else if(!strcmp(function, "pois"))
@@ -1226,7 +1352,9 @@ static struct command_table commands[] = {
 	{"waypoints",command_cast(gui_internal_cmd2)},
 	{"write",command_cast(gui_internal_cmd_write)},
 	{"about",command_cast(gui_internal_cmd2)},
-
+#if HAS_IFADDRS
+	{"network_info",command_cast(gui_internal_cmd2)},
+#endif
 };
 
 void
