@@ -41,6 +41,8 @@ struct selector {
 };
 static enum item_type selectors_BankTypes[]={type_poi_bank,type_poi_bank, type_poi_atm,type_poi_atm, type_none};
 static enum item_type selectors_FuelTypes[]={type_poi_fuel,type_poi_fuel,type_none};
+static enum item_type selectors_BusTrainTypes[]={type_poi_rail_station,type_poi_rail_station,
+    type_poi_rail_halt,type_poi_rail_tram_stop,type_poi_bus_station,type_poi_bus_stop,type_none};
 static enum item_type selectors_HotelTypes[]={type_poi_hotel,type_poi_camp_rv,type_poi_camping,type_poi_camping,
     type_poi_resort,type_poi_resort,type_poi_motel,type_poi_hostel,type_none};
 static enum item_type selectors_RestaurantTypes[]={type_poi_bar,type_poi_picnic,type_poi_burgerking,type_poi_fastfood,
@@ -67,6 +69,7 @@ static enum item_type selectors_OtherTypes[]={type_point_unspecified,type_poi_la
 struct selector selectors[]={
 	{"bank","Bank",selectors_BankTypes},
 	{"fuel","Fuel",selectors_FuelTypes},
+	{"bus_stop","Bus&Train",selectors_BusTrainTypes},
 	{"hotel","Hotel",selectors_HotelTypes},
 	{"restaurant","Restaurant",selectors_RestaurantTypes},
 	{"shopping","Shopping",selectors_ShoppingTypes},
@@ -533,6 +536,8 @@ gui_internal_cmd_pois_filter(struct gui_priv *this, struct widget *wm, void *dat
 	
 	if (this->keyboard)
 		gui_internal_widget_append(w, gui_internal_keyboard(this, keyboard_mode));
+	else
+		gui_internal_keyboard_show_native(this, w, keyboard_mode, getenv("LANG"));
 	gui_internal_menu_render(this);
 
 
