@@ -192,14 +192,22 @@ gui_internal_find_next_possible_key(char *search_text, char *wm_name, char *poss
 {
 	gchar* trunk_name;
 	if (item_name) {
-		trunk_name = g_strrstr(item_name, search_text);
+	
+		trunk_name = linguistics_expand_special(item_name,1);
+		if(!trunk_name){
+			trunk_name = g_strrstr(item_name, search_text);
+		}
+		
 		if (trunk_name) {
-			char next_char = trunk_name[strlen(search_text)];
+			int next_char_pos = strlen(search_text);
+			char next_char = trunk_name[next_char_pos];
 			int i;
 			int len = strlen(possible_keys);
+			
 			for(i = 0; (i<len) && (possible_keys[i] != next_char) ;i++) ;
-			if (i==len || !len) {
-				possible_keys[len]=trunk_name[strlen(search_text)];
+			
+			if ((i==len || !len) && !strncmp(trunk_name, search_text, next_char_pos)) {
+				possible_keys[len]=trunk_name[next_char_pos];
 				possible_keys[len+1]='\0';
 			}
 			dbg(lvl_info,"searching for %s, found: %s, currently possible_keys: %s \n", search_text, item_name, possible_keys);
@@ -211,6 +219,10 @@ static void
 gui_internal_highlight_possible_keys(struct gui_priv *this, char *possible_keys)
 {
 	struct menu_data *md;
+	int first_available_key_found = 0;
+
+	if (!this->keyboard)
+		return;
 
 	md=gui_internal_menu_data(this);
 	if (md && md->keyboard && !(this->flags & 2048)) {
@@ -222,12 +234,30 @@ gui_internal_highlight_possible_keys(struct gui_priv *this, char *possible_keys)
 			while (lk2) {
 				struct widget *child_=lk2->data;
 				lk2=g_list_next(lk2);
-				if (child_->data && strcmp("\b", child_->data)) { // FIXME don't disable special keys
+				// The data_free part is an evil hack based on the observation that
+				// regular keys have set it to non-NULL whereas special keys appear
+				// to have it set to NULL.
+				if (child_->data && strcmp("\b", child_->data) && child_->data_free) {
 					if ( (strlen(possible_keys) == 0) ||
 					     (g_strrstr(possible_keys, child_->data)!=NULL ) ) {
-						child_->state|= STATE_HIGHLIGHTED|STATE_VISIBLE|STATE_SENSITIVE|STATE_CLEAR ;
+						if(this->hide_keys){
+							child_->state|= STATE_SENSITIVE|STATE_CLEAR ;
+							child_->state&= ~(STATE_INVISIBLE);
+						}else{
+							child_->state|= STATE_SENSITIVE|STATE_CLEAR|STATE_HIGHLIGHTED ;
+						}
+						// Select and highlight the first possible button.
+						if (!first_available_key_found) {
+							gui_internal_highlight_do(this, child_);
+							first_available_key_found=1;
+						}
 					} else {
-						child_->state&= ~(STATE_HIGHLIGHTED|STATE_VISIBLE|STATE_SELECTED) ;
+						if(this->hide_keys){
+							child_->state&= ~(STATE_SELECTED|STATE_SENSITIVE) ;
+							child_->state|= STATE_INVISIBLE;
+						}else{
+							child_->state&= ~(STATE_HIGHLIGHTED);
+						}
 					}
 					gui_internal_widget_render(this,child_);
 				}
@@ -431,13 +461,9 @@ gui_internal_search_changed(struct gui_priv *this, struct widget *wm, void *data
 			search_attr.type=attr_house_number;
 		search_attr.u.str=wm->text;
 		search_list_search(this->sl, &search_attr, 1);
-		// Text is not necessarily entered via the on-screen keyboard,
-		// but we now switch it to lower case anyway.
-		gui_internal_keyboard_to_lower_case(this);
 		gui_internal_search_idle_start(this, wm->name, search_list, param);
 	} else {
 		// If not enough content is entered, we highlight all keys.
-		gui_internal_keyboard_to_upper_case(this);
 		gui_internal_highlight_possible_keys(this, "");
 	}
 	l=g_list_last(this->root.children);
@@ -570,6 +596,8 @@ gui_internal_search(struct gui_priv *this, const char *what, const char *type, i
 	wk->name=g_strdup(type);
 	if (this->keyboard)
 		gui_internal_widget_append(w, gui_internal_keyboard(this, keyboard_mode));
+	else
+		gui_internal_keyboard_show_native(this, w, keyboard_mode, getenv("LANG"));
 	gui_internal_menu_render(this);
 }
 
