@@ -18,6 +18,8 @@
  */
  
 /** @file vehicle.c
+ * @defgroup vehicle-plugins vehicle plugins
+ * @ingroup plugins
  * @brief Generic components of the vehicle object.
  * 
  * This file implements the generic vehicle interface, i.e. everything which is
@@ -52,6 +54,7 @@
 #include "vehicle.h"
 #include "navit_nls.h"
 
+
 struct vehicle {
 	NAVIT_OBJECT
 	struct vehicle_methods meth;
@@ -122,7 +125,7 @@ vehicle_new(struct attr *parent, struct attr **attrs)
 		*colon = '\0';
 	dbg(lvl_debug, "source='%s' type='%s'\n", source->u.str, type);
 
-	vehicletype_new = plugin_get_vehicle_type(type);
+	vehicletype_new = plugin_get_category_vehicle(type);
 	if (!vehicletype_new) {
 		dbg(lvl_error, "invalid source '%s': unknown type '%s'\n", source->u.str, type);
 		g_free(type);
