@@ -1,4 +1,4 @@
-/**
+/*
  * Navit, a modular navigation system.
  * Copyright (C) 2005-2008 Navit Team
  *
@@ -59,6 +59,14 @@ int inet_aton(const char *cp, struct in_addr *inp)
 }
 #endif
 
+/**
+ * @defgroup vehicle-file Vehicle File
+ * @ingroup vehicle-plugins
+ * @brief The Vehicle to gain position data from a file, pipe, serial interface or a socket
+ *
+ * @{
+ */
+
 static void vehicle_file_disable_watch(struct vehicle_priv *priv);
 static void vehicle_file_enable_watch(struct vehicle_priv *priv);
 static int vehicle_file_parse(struct vehicle_priv *priv, char *buffer);
@@ -84,7 +92,7 @@ struct vehicle_priv {
 	char *source;
 	struct callback_list *cbl;
 	int fd;
-	struct callback *cb,*cbt;
+	struct callback *cb,*cbt,*cb_fix_timeout;
 	char *buffer;
 	int buffer_pos;
 	char *nmea_data;
@@ -114,6 +122,7 @@ struct vehicle_priv {
 	enum file_type file_type;
 	FILE *file;
 	struct event_watch *watch;
+	struct event_timeout *ev_fix_timeout;
 	speed_t baudrate;
 	struct attr ** attrs;
 	char fixiso8601[128];
@@ -129,16 +138,13 @@ struct vehicle_priv {
 	int process_statefile;
 };
 
-//***************************************************************************
-/** @fn static int vehicle_win32_serial_track(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: Callback of the plugin
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
-* @return     always 1
-*****************************************************************************
-**/
+/**
+* @brief Callback of the plugin
+*
+* @param priv Pointer on the private data of the plugin
+*
+* @return Always 1
+*/
 #ifdef _WIN32
 static int vehicle_win32_serial_track(struct vehicle_priv *priv)
 {
@@ -207,17 +213,13 @@ static int vehicle_win32_serial_track(struct vehicle_priv *priv)
 }
 #endif
 
-//***************************************************************************
-/** @fn static int vehicle_file_open(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: open dialogue with the GPS
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
-* @return     1 if ok
-*             0 if error
-*****************************************************************************
-**/
+/**
+* @brief Open dialogue with the GPS
+*
+* @param priv Pointer on the private data of the plugin
+*
+* @return 1 if ok, 0 if error
+*/
 static int
 vehicle_file_open(struct vehicle_priv *priv)
 {
@@ -287,7 +289,11 @@ vehicle_file_open(struct vehicle_priv *priv)
 		p=strchr(p,':');
 		if (p) {
 			p++;
-			write(priv->fd, p, strlen(p));
+			int write_result;
+			write_result = write(priv->fd, p, strlen(p));
+			if (write_result == -1){
+				dbg(lvl_warning, "write failed.");
+			}
 		}
 		priv->file_type = file_type_socket;
 #endif //HAVE_SOCKET
@@ -318,14 +324,11 @@ vehicle_file_open(struct vehicle_priv *priv)
     return(priv->fd != -1);
 }
 
-//***************************************************************************
-/** @fn static void vehicle_file_close(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: close dialogue with the GPS
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
-**/
+/**
+* @brief Close dialogue with the GPS
+*
+* @param priv Pointer on the private data of the plugin
+*/
 static void
 vehicle_file_close(struct vehicle_priv *priv)
 {
@@ -356,16 +359,13 @@ vehicle_file_close(struct vehicle_priv *priv)
     }
 }
 
-//***************************************************************************
-/** @fn static int vehicle_file_enable_watch_timer(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: Enable watch timer to get GPS data
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
-* @return     always 0
-*****************************************************************************
-**/
+/**
+* @brief Enable watch timer to get GPS data
+*
+* @param priv Pointer on the private data of the plugin
+*
+* @return Always 0
+*/
 static int
 vehicle_file_enable_watch_timer(struct vehicle_priv *priv)
 {
@@ -376,19 +376,44 @@ vehicle_file_enable_watch_timer(struct vehicle_priv *priv)
 }
 
 
-//***************************************************************************
-/** @fn static int vehicle_file_parse( struct vehicle_priv *priv,
-*                                      char *buffer)
-*****************************************************************************
-* @b Description: Parse the buffer
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-* @param      buffer : data buffer (null terminated)
-*****************************************************************************
-* @return     1 if The GPRMC Sentence is found
-*             0 if not found
-*****************************************************************************
-**/
+/**
+* @brief This is a callback function, called when the fix timeout
+*        is done. Set the position to invalid.
+*
+* @param priv Pointer on the private data of the plugin
+*/
+static void
+vehicle_file_fix_timeout_cb(struct vehicle_priv *priv)
+{
+	priv->valid = attr_position_valid_invalid;
+	priv->ev_fix_timeout = NULL;
+	callback_list_call_attr_0(priv->cbl, attr_position_coord_geo);
+}
+
+
+/**
+* @brief Cancel previous fix timeout event and add a new one
+*
+* @param priv Pointer on the private data of the plugin
+*/
+static void
+vehicle_file_restart_fix_timeout(struct vehicle_priv *priv)
+{
+	if (priv->ev_fix_timeout != NULL)
+		event_remove_timeout(priv->ev_fix_timeout);
+	priv->ev_fix_timeout = event_add_timeout(10000, 0, priv->cb_fix_timeout);
+}
+
+
+/**
+* @brief Parse the buffer
+*
+* @param priv Pointer on the private data of the plugin
+* @param buffer Data buffer (null terminated)
+*
+* @return 1 if new coords were received (fixtime changed) or changed to invalid,
+*         0 if not found
+*/
 static int
 vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 {
@@ -451,14 +476,14 @@ vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 		*p++ = '\0';
 	}
 
-	if (!strncmp(buffer, "$GPGGA", 6)) {
+	if (!strncmp(&buffer[3], "GGA", 3)) {
 		/*                                                           1 1111
 		   0      1          2         3 4          5 6 7  8   9     0 1234
 		   $GPGGA,184424.505,4924.2811,N,01107.8846,E,1,05,2.5,408.6,M,,,,0000*0C
 		   UTC of Fix[1],Latitude[2],N/S[3],Longitude[4],E/W[5],Quality(0=inv,1=gps,2=dgps)[6],Satelites used[7],
 		   HDOP[8],Altitude[9],"M"[10],height of geoid[11], "M"[12], time since dgps update[13], dgps ref station [14]
 		 */
-		if (*item[2] && *item[3] && *item[4] && *item[5]) {
+		if (*item[2] && *item[3] && *item[4] && *item[5] && *item[6] > 0) {
 			lat = g_ascii_strtod(item[2], NULL);
 			priv->geo.lat = floor(lat / 100);
 			lat -= priv->geo.lat * 100;
@@ -474,19 +499,31 @@ vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 
 			if (!g_ascii_strcasecmp(item[5],"W"))
 				priv->geo.lng=-priv->geo.lng;
-			priv->valid=attr_position_valid_valid;
-            dbg(lvl_info, "latitude '%2.4f' longitude %2.4f\n", priv->geo.lat, priv->geo.lng);
 
-		} else
+			if (priv->valid == attr_position_valid_invalid)
+				ret = 1;
+			priv->valid = attr_position_valid_valid;
+			vehicle_file_restart_fix_timeout(priv);
+
+			if (*item[1] && strncmp(priv->fixtime, item[1], sizeof(priv->fixtime))) {
+				ret = 1;
+				strncpy(priv->fixtime, item[1], sizeof(priv->fixtime));
+			}
+
+			dbg(lvl_info, "latitude '%2.4f' longitude %2.4f\n", priv->geo.lat, priv->geo.lng);
+
+		} else {
+			if (priv->valid == attr_position_valid_valid)
+				ret = 1;
 			priv->valid=attr_position_valid_invalid;
+		}
+
 		if (*item[6])
 			sscanf(item[6], "%d", &priv->status);
 		if (*item[7])
-		sscanf(item[7], "%d", &priv->sats_used);
+			sscanf(item[7], "%d", &priv->sats_used);
 		if (*item[8])
 			sscanf(item[8], "%lf", &priv->hdop);
-		if (*item[1]) 
-			strncpy(priv->fixtime, item[1], sizeof(priv->fixtime));
 		if (*item[9])
 			sscanf(item[9], "%lf", &priv->height);
 
@@ -500,7 +537,7 @@ vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 			}
 		}
 	}
-	if (!strncmp(buffer, "$GPVTG", 6)) {
+	if (!strncmp(&buffer[3], "VTG", 3)) {
 		/* 0      1      2 34 5    6 7   8
 		   $GPVTG,143.58,T,,M,0.26,N,0.5,K*6A
 		   Course Over Ground Degrees True[1],"T"[2],Course Over Ground Degrees Magnetic[3],"M"[4],
@@ -516,7 +553,7 @@ vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 			dbg(lvl_info,"direction %lf, speed %2.1lf\n", priv->direction, priv->speed);
 		}
 	}
-	if (!strncmp(buffer, "$GPRMC", 6)) {
+	if (!strncmp(&buffer[3], "RMC", 3)) {
 		/*                                                           1     1
 		   0      1      2 3        4 5         6 7     8     9      0     1
 		   $GPRMC,123519,A,4807.038,N,01131.000,E,022.4,084.4,230394,003.1,W*6A
@@ -536,8 +573,40 @@ vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 				&priv->fixmonth,
 				&priv->fixyear);
 			priv->fixyear += 2000;
+
+			lat = g_ascii_strtod(item[3], NULL);
+			priv->geo.lat = floor(lat / 100);
+			lat -= priv->geo.lat * 100;
+			priv->geo.lat += lat / 60;
+
+			if (!g_ascii_strcasecmp(item[4],"S"))
+				priv->geo.lat=-priv->geo.lat;
+
+			lng = g_ascii_strtod(item[5], NULL);
+			priv->geo.lng = floor(lng / 100);
+			lng -= priv->geo.lng * 100;
+			priv->geo.lng += lng / 60;
+
+			if (!g_ascii_strcasecmp(item[6],"W"))
+				priv->geo.lng=-priv->geo.lng;
+
+			if (priv->valid == attr_position_valid_invalid)
+				ret = 1;
+			priv->valid=attr_position_valid_valid;
+			vehicle_file_restart_fix_timeout(priv);
+
+			if (*item[1] && strncmp(priv->fixtime, item[1], sizeof(priv->fixtime))) {
+				ret = 1;
+				strncpy(priv->fixtime, item[1], sizeof(priv->fixtime));
+			}
+
+			dbg(lvl_info, "latitude '%2.4f' longitude %2.4f\n", priv->geo.lat, priv->geo.lng);
+
+		} else {
+			if (priv->valid == attr_position_valid_valid)
+				ret = 1;
+			priv->valid=attr_position_valid_invalid;
 		}
-		ret = 1;
 	}
 	if (!strncmp(buffer, "$GPGSV", 6) && i >= 4) {
 	/*
@@ -576,7 +645,7 @@ vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 			priv->next_count=0;
 		}
 	}
-	if (!strncmp(buffer, "$GPZDA", 6)) {
+	if (!strncmp(&buffer[3], "ZDA", 3)) {
 	/*
 		0        1        2  3  4    5  6
 		$GPZDA,hhmmss.ss,dd,mm,yyyy,xx,yy*CC
@@ -609,16 +678,11 @@ vehicle_file_parse(struct vehicle_priv *priv, char *buffer)
 	return ret;
 }
 
-//***************************************************************************
-/** @fn static void vehicle_file_io(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: function to get data from GPS
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
-* @remarks 
-*****************************************************************************
-**/
+/**
+* @brief Function to get data from GPS
+*
+* @param  priv Pointer on the private data of the plugin
+*/
 static void
 vehicle_file_io(struct vehicle_priv *priv)
 {
@@ -684,14 +748,11 @@ vehicle_file_io(struct vehicle_priv *priv)
 		callback_list_call_attr_0(priv->cbl, attr_position_coord_geo);
 }
 
-//***************************************************************************
-/** @fn static void vehicle_file_enable_watch(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: Enable watch
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
-**/
+/**
+* @brief Enable watch
+*
+* @param priv Pointer on the private data of the plugin
+*/
 static void
 vehicle_file_enable_watch(struct vehicle_priv *priv)
 {
@@ -712,14 +773,11 @@ vehicle_file_enable_watch(struct vehicle_priv *priv)
     }
 }
 
-//***************************************************************************
-/** @fn static void vehicle_file_disable_watch(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: Disable watch
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
-**/
+/**
+* @brief Disable watch
+*
+* @param priv Pointer on the private data of the plugin
+*/
 static void
 vehicle_file_disable_watch(struct vehicle_priv *priv)
 {
@@ -741,16 +799,13 @@ vehicle_file_disable_watch(struct vehicle_priv *priv)
     }
 }
 
-//***************************************************************************
-/** @fn static void vehicle_priv vehicle_file_destroy(struct vehicle_priv *priv)
-*****************************************************************************
-* @b Description: Function called to uninitialize the plugin
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-*****************************************************************************
+/**
+* @brief Function called to uninitialize the plugin
+*
+* @param priv Pointer on the private data of the plugin
+*
 * @remarks private data is freed by this function (g_free)
-*****************************************************************************
-**/
+*/
 static void
 vehicle_file_destroy(struct vehicle_priv *priv)
 {
@@ -780,21 +835,15 @@ vehicle_file_destroy(struct vehicle_priv *priv)
 	g_free(priv);
 }
 
-//***************************************************************************
-/** @fn static int vehicle_file_position_attr_get(struct vehicle_priv *priv,
-*                                                 enum attr_type type,
-*                                                 struct attr *attr)
-*****************************************************************************
-* @b Description: Function called to get attribute
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-* @param      type : attribute type called
-* @param      attr : structure to return the attribute value
-*****************************************************************************
-* @return     1 if ok
-*             0 for unkown or invalid attribute
-*****************************************************************************
-**/
+/**
+* @brief Function called to get attribute
+*
+* @param priv Pointer on the private data of the plugin
+* @param type Attribute type called
+* @param attr Structure to return the attribute value
+*
+* @return 1 if ok, 0 for unknown or invalid attribute
+*/
 static int
 vehicle_file_position_attr_get(struct vehicle_priv *priv,
 			       enum attr_type type, struct attr *attr)
@@ -864,21 +913,15 @@ vehicle_file_position_attr_get(struct vehicle_priv *priv,
 	return 1;
 }
 
-//***************************************************************************
-/** @fn static int vehicle_file_sat_attr_get(struct vehicle_priv *priv,
-*                                                 enum attr_type type,
-*                                                 struct attr *attr)
-*****************************************************************************
-* @b Description: Function called to get satellite attribute
-*****************************************************************************
-* @param      priv : pointer on the private data of the plugin
-* @param      type : attribute type called
-* @param      attr : structure to return the attribute value
-*****************************************************************************
-* @return     1 if ok
-*             0 for unkown attribute
-*****************************************************************************
-**/
+/**
+* @brief Function called to get satellite attribute
+*
+* @param priv Pointer on the private data of the plugin
+* @param type Attribute type called
+* @param attr Structure to return the attribute value
+*
+* @return 1 if ok, 0 for unknown attribute
+*/
 static int
 vehicle_file_sat_attr_get(void *priv_data, enum attr_type type, struct attr *attr)
 {
@@ -921,23 +964,17 @@ static struct vehicle_methods vehicle_file_methods = {
 	vehicle_file_position_attr_get,
 };
 
-//***************************************************************************
-/** @fn static struct vehicle_priv * vehicle_file_new_file(
-*                                       struct vehicle_methods *meth,
-*                                       struct callback_list   *cbl,
-*                                       struct attr            **attrs)
-*****************************************************************************
-* @b Description: Function called to initialize the plugin
-*****************************************************************************
-* @param      meth  : ?
-* @param      cbl   : ?
-* @param      attrs : ?
-*****************************************************************************
-* @return     pointer on the private data of the plugin
-*****************************************************************************
-* @remarks private data is allocated by this function (g_new0)
-*****************************************************************************
-**/
+/**
+* @brief Function called to initialize the plugin
+*
+* @param meth ?
+* @param cbl ?
+* @param attrs ?
+*
+* @return Pointer on the private data of the plugin
+*
+* @remarks Private data is allocated by this function (g_new0)
+*/
 static struct vehicle_priv *
 vehicle_file_new_file(struct vehicle_methods
 		      *meth, struct callback_list
@@ -957,7 +994,7 @@ vehicle_file_new_file(struct vehicle_methods
 	if(source == NULL){
 		 dbg(lvl_error,"Missing source attribute");
 		 return NULL;
-    }
+	}
 	ret = g_new0(struct vehicle_priv, 1);   // allocate and initialize to 0
 	ret->fd = -1;
 	ret->cbl = cbl;
@@ -965,6 +1002,8 @@ vehicle_file_new_file(struct vehicle_methods
 	ret->buffer = g_malloc(buffer_size);
 	ret->time=1000;
 	ret->baudrate=B4800;
+	ret->fixtime[0] = '\0';
+	ret->ev_fix_timeout = NULL;
 	state_file=attr_search(attrs, NULL, attr_state_file);
 	if (state_file) 
 		ret->statefile=g_strdup(state_file->u.str);
@@ -1013,6 +1052,7 @@ vehicle_file_new_file(struct vehicle_methods
 	*meth = vehicle_file_methods;
 	ret->cb=callback_new_1(callback_cast(vehicle_file_io), ret);
 	ret->cbt=callback_new_1(callback_cast(vehicle_file_enable_watch_timer), ret);
+	ret->cb_fix_timeout=callback_new_1(callback_cast(vehicle_file_fix_timeout_cb), ret);
 	if (ret->statefile && file_exists(ret->statefile)) {
 		ret->process_statefile=1;
 		event_add_timeout(1000, 0, ret->cb);
@@ -1037,17 +1077,14 @@ vehicle_file_new_file(struct vehicle_methods
 	return ret;
 }
 
-//***************************************************************************
-/** @fn void plugin_init(void)
-*****************************************************************************
-* @b Description: Initialisation of vehicle_file plugin
-*****************************************************************************
-**/
+/**
+* @brief Initialisation of vehicle_file plugin
+*/
 void plugin_init(void)
 {
 	dbg(lvl_debug, "vehicle_file:plugin_init:enter\n");
-	plugin_register_vehicle_type("file", vehicle_file_new_file);
-	plugin_register_vehicle_type("pipe", vehicle_file_new_file);
-	plugin_register_vehicle_type("socket", vehicle_file_new_file);
-	plugin_register_vehicle_type("serial", vehicle_file_new_file);
+	plugin_register_category_vehicle("file", vehicle_file_new_file);
+	plugin_register_category_vehicle("pipe", vehicle_file_new_file);
+	plugin_register_category_vehicle("socket", vehicle_file_new_file);
+	plugin_register_category_vehicle("serial", vehicle_file_new_file);
 }
